/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */

#include "config.h"
#include <stdio.h>
#include <netinet/in.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <assert.h>
#include <X11/Xlib.h>
#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/Frame.h>
#include <Xm/BulletinB.h>
#include "xmap.h"
#include "mode.h"
#include "debug.h"
#include "vector/vector.h"

extern Widget draw_w;
extern Widget toplevel;

static Widget dialog = NULL;
static Widget bul;
static Widget ok;
static Widget cancel;
static Widget help;
static Widget placename;
static Widget l1;

static int answer = ANS_NONE;

static void ok_cb(Widget dialog, XtPointer client_data, XtPointer call_data)
{
    answer = ANS_OK;
}

static void cancel_cb(Widget dialog, XtPointer client_data, XtPointer call_data)
{
    answer = ANS_CANCEL;
}

#define BY 80

int placedialog(char *title, char *result)
{
    Arg args[100];
    int n;
    Pixel foreground, background;
    XtAppContext app = XtWidgetToApplicationContext(toplevel);
    XmString s;

    *result = '\0';
    if (!dialog) {

	foreground = get_pixel(draw_w, "Black");
	background = get_pixel(draw_w, "Snow");
	dialog = XtVaCreatePopupShell("place",
			   xmDialogShellWidgetClass, GetTopShell(draw_w),
				      XmNdeleteResponse, XmUNMAP,
				      XmNresizePolicy, XmRESIZE_NONE,
				      XmNnoResize, True,
				    //  XmNwidth, 290,
				    //  XmNminWidth, 290,
				    //  XmNmaxWidth, 290,
				    //  XmNheight, 130,
				    //  XmNminHeight, 130,
				    //  XmNmaxHeight, 130,
				      NULL);

	bul = XtVaCreateWidget("bul",
			       xmBulletinBoardWidgetClass, dialog,
			       XmNforeground, foreground,
			       XmNautoUnmanage, False,
			       XmNdialogStyle, XmDIALOG_WORK_AREA,
			       NULL);

	XtAddCallback(bul, XmNunmapCallback, (void *) cancel_cb, NULL);

	l1 = XtVaCreateManagedWidget("l1", xmLabelWidgetClass, bul,
				     XmNy, 35,
				     NULL);

	placename = XtVaCreateManagedWidget("t1", xmTextFieldWidgetClass, bul,
					    XmNhighlightThickness, 0,
					    XmNshadowThickness, 1,
					    XmNmarginWidth, 2,
					    XmNmarginHeight, 2,
					    XmNbackground, background,
					    XmNx, 60,
					    XmNy, 33,
					    XmNwidth, 200,
					    NULL);

	cancel = XtVaCreateManagedWidget("Cancel",
					 xmPushButtonWidgetClass, bul,
					 XmNforeground, foreground,
					 XmNhighlightThickness, 1,
					 XmNwidth, 60,
					 XmNx, 100,
					 XmNy, BY,
					 NULL);
	XtAddCallback(cancel, XmNactivateCallback, cancel_cb, NULL);

	ok = XtVaCreateManagedWidget("OK",
				     xmPushButtonWidgetClass, bul,
				     XmNforeground, foreground,
				     XmNhighlightThickness, 1,
				     XmNwidth, 60,
				     XmNx, 20,
				     XmNy, BY,
				     NULL);
	XtAddCallback(ok, XmNactivateCallback, ok_cb, NULL);

	help = XtVaCreateManagedWidget("Help",
				       xmPushButtonWidgetClass, bul,
				       XmNforeground, foreground,
				       XmNhighlightThickness, 1,
				       XmNwidth, 60,
				       XmNx, 180,
				       XmNy, BY,
				       NULL);
	XtAddCallback(help, XmNactivateCallback, show_help_cb, "loc_ll.html");
    }
    s = XmStringCreateSimple("Name:");
    XtVaSetValues(l1, XmNlabelString, s, NULL);
    XmStringFree(s);

    XtManageChild(bul);

    XtVaSetValues(dialog,
		  XmNtitle, title,
		  NULL);
    XtManageChild(dialog);
    XtPopup(dialog, XtGrabNone);

    answer = ANS_NONE;

    while (answer == ANS_NONE)
	XtAppProcessEvent(app, XtIMAll);

    XtPopdown(dialog);

    if (answer == ANS_OK) {
	strcpy(result, XmTextFieldGetString(placename));
    }
    /* make sure the dialog goes away before returning. Sync with server
     * and update the display.
     */
    XSync(XtDisplay(dialog), 0);
    XmUpdateDisplay(toplevel);

    return (answer);
}


void psearch(char *name)
{
    int places, soundexcode, s;
    void *pstart;
    char *p;
    Place pl;
    long psize;
    struct stat statbuf;
    char codei[6];
    char codeo[6];
    char bufp[256];
    extern char *soundex();

    snprintf(bufp, 255, "%s/places.dat", get_vmap_root());
    places = open(bufp, O_RDONLY);
    if (fstat(places, &statbuf))
	return;
    psize = statbuf.st_size;

    pstart = mmap(0, psize, PROT_READ, MAP_SHARED, places, 0);

    (Place *) p = pstart + 64;	// skip header

    soundex(name, codei);

    while (1) {
	if (!(p < (char *)(pstart + psize)))
	    break;
	pl.lat = ntohl(MY_ALIGN(&((Place *) p)->lat));
	pl.lon = ntohl(MY_ALIGN(&((Place *) p)->lon));
	pl.st[0] = ((Place *)p)->st[0];
	pl.st[1] = ((Place *)p)->st[1];
	memcpy(pl.name, ((Place *) p)->name, ((Place *) p)->len - 15);
	pl.name[((Place *) p)->len - 15] = '\0';

	soundex(pl.name, codeo);
	if (strncmp(codei, codeo, 6) == 0) {
	    snprintf(bufp, 255, "%d %d  %s, %c%c\n", pl.lat, pl.lon, pl.name, pl.st[0], pl.st[1]);
	    error(bufp, 0);
	}	
	p += ((Place *) p)->len;
    }
    munmap(pstart, psize);
    close(places);
}

void findplace(void)
{
    char result[256];

    if (placedialog("Search", result) == ANS_OK) {
fprintf(stderr, "result = \"%s\"\n", result);
	if (*result) psearch(result);
    }
}
