/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */
#include "config.h"

#include <X11/Xos.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/FileSB.h>
#include <Xm/Frame.h>
#include <Xm/MainW.h>
#include <Xm/RowColumn.h>
#include <Xm/LabelG.h>
#include <Xm/Form.h>
#include <Xm/List.h>
#include <Xm/Label.h>
#include <Xm/ScrollBar.h>
#include <Xm/DrawnB.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <stdio.h>
#include <string.h>
#include <sys/param.h>

#include "xmap.h"
#include "debug.h"

#define WIDTH 28

extern Widget toplevel;
extern Pixmap stipple;
extern Widget draww;

static void tool_cb();
extern XtAppContext app;
extern Widget toolbar, status_1, main_w;
extern XtTranslations button_translations;
enum {
    WAIT, DISPLAY, IGNORE
};
static int bs;
static XtIntervalId id;

static void unshow_tip(void);

Widget showToolBarButton(int button, int ord, Pixmap * pmap, void *p, XtPointer p_data)
{
    Widget w;
    GC hlGC;

    w = XtVaCreateWidget("tool", xmDrawnButtonWidgetClass, toolbar,
			 XmNlabelType, XmPIXMAP,
			 XmNlabelPixmap, pmap,
			 XmNhighlightThickness, 0,
			 XmNshadowThickness, 0,
			 XmNpushButtonEnabled, True,
			 XmNshadowType, XmSHADOW_ETCHED_IN,
		       XmNx, 6 + (ord * WIDTH) + tbutton[button].spacing,
			 XmNy, 3,
			 NULL);

    XtAddCallback(w, XmNactivateCallback, tool_cb, NULL);
    if (p)
      XtAddCallback(w, XmNactivateCallback, (void *)p, (XtPointer) p_data);
    tbutton[button].w = w;
    XtOverrideTranslations(w, button_translations);
    XtManageChild(w);
    return w;
}

static void to_cb(Widget w)
{
    char bufp[256];

    id = 0;
    snprintf(bufp, 255, "%s", get_tip(w));
    if (!options.toolhelp)
	XmTextFieldSetString(status_1, bufp);
    else
	do_tip(toplevel, w, bufp, 60, &options);

    bs = DISPLAY;
}

void MyMotionAction(Widget w, XMotionEvent * e, String * p, Cardinal n)
{
    char bufp[256];

    if (bs == WAIT) {
	if (id) XtRemoveTimeOut(id);
	id = XtAppAddTimeOut(app, 550L, (XtTimerCallbackProc) to_cb, w);
    }
#ifdef REMOVE_WIGGLE
    else if (bs == DISPLAY) {
	bs = IGNORE;
	if (!options.toolhelp) {
	    snprintf(bufp, 255, "");
	    XmTextFieldSetString(status_1, bufp);
	} else
	    unshow_tip(); //tf
    }
#endif
//tf    bs = IGNORE;
}

void MyEnterAction(Widget w, XEvent * e, String * p, Cardinal n)
{
    bs = WAIT;
    XtVaSetValues(w, XmNshadowThickness, 1, NULL);
    if (id) XtRemoveTimeOut(id);
    id = XtAppAddTimeOut(app, 550L, (XtTimerCallbackProc) to_cb, w);
}

void MyLeaveAction(Widget w, XEvent * e, String * p, Cardinal n)
{
    char bufp[256];

    if (bs == WAIT && id) {
	XtRemoveTimeOut(id);
	id = 0;
    }
    XtVaSetValues(w, XmNshadowThickness, 0, NULL);
    if (bs == DISPLAY) {
	if (!options.toolhelp) {
	    sprintf(bufp, "");
	    XmTextFieldSetString(status_1, bufp);
	} else
	    unshow_tip(); //tf
    }
//tf    bs = IGNORE;
}


static void unshow_tip()
{
    char bufp[256];

    if (bs == WAIT && id) {
	XtRemoveTimeOut(id);
	id = 0;
    }
    if (bs == DISPLAY) {
	if (!options.toolhelp) {
	    sprintf(bufp, "");
	    XmTextFieldSetString(status_1, bufp);
	} else
	    undo_tip(&options);
    }
    bs = IGNORE;
}

static void tool_cb(Widget w, XtPointer client_data, XmToggleButtonCallbackStruct * state)
{
    if (options.toolhelp)
	unshow_tip();
    /*
    if (tbutton[(int) client_data].func)
	tbutton[(int) client_data].func(w, NULL);
    */
}
